@@ -4,6 +4,13 @@ fn main() {
     let password = Password::with_theme(&ColorfulTheme::default())
         .with_prompt("Password")
         .with_confirmation("Repeat password", "Error: the passwords don't match.")
+        .validate_with(|input: &String| -> Result<(), &str> {
+            if input.len() > 3 {
+                Ok(())
+            } else {
+                Err("Password must be longer than 3")
+            }
+        })
         .interact()
         .unwrap();
 
