@@ -1,4 +1,4 @@
-use std::{fmt::Debug, io, iter, str::FromStr};
+use std::{cmp::Ordering, fmt::Debug, io, iter, str::FromStr};
 
 #[cfg(feature = "completion")]
 use crate::completion::Completion;
@@ -11,6 +11,8 @@ use crate::{
 
 use console::{Key, Term};
 
+type ValidatorCallback<'a, T> = Box<dyn FnMut(&T) -> Option<String> + 'a>;
+
 /// Renders an input prompt.
 ///
 /// ## Example usage
@@ -39,13 +41,14 @@ use console::{Key, Term};
 /// ```
 pub struct Input<'a, T> {
     prompt: String,
+    post_completion_text: Option<String>,
     report: bool,
     default: Option<T>,
     show_default: bool,
     initial_text: Option<String>,
     theme: &'a dyn Theme,
     permit_empty: bool,
-    validator: Option<Box<dyn FnMut(&T) -> Option<String> + 'a>>,
+    validator: Option<ValidatorCallback<'a, T>>,
     #[cfg(feature = "history")]
     history: Option<&'a mut dyn History<T>>,
     #[cfg(feature = "completion")]
@@ -70,6 +73,15 @@ impl<T> Input<'_, T> {
         self
     }
 
+    /// Changes the prompt text to the post completion text after input is complete
+    pub fn with_post_completion_text<S: Into<String>>(
+        &mut self,
+        post_completion_text: S,
+    ) -> &mut Self {
+        self.post_completion_text = Some(post_completion_text.into());
+        self
+    }
+
     /// Indicates whether to report the input value after interaction.
     ///
     /// The default is to report the input value.
@@ -119,6 +131,7 @@ impl<'a, T> Input<'a, T> {
     pub fn with_theme(theme: &'a dyn Theme) -> Self {
         Self {
             prompt: "".into(),
+            post_completion_text: None,
             report: true,
             default: None,
             show_default: true,
@@ -262,7 +275,7 @@ where
         loop {
             let default_string = self.default.as_ref().map(ToString::to_string);
 
-            render.input_prompt(
+            let prompt_len = render.input_prompt(
                 &self.prompt,
                 if self.show_default {
                     default_string.as_deref()
@@ -270,7 +283,6 @@ where
                     None
                 },
             )?;
-            term.flush()?;
 
             // Read input by keystroke so that we can suppress ascii control characters
             if !term.features().is_attended() {
@@ -287,19 +299,35 @@ where
                 chars = initial.chars().collect();
                 position = chars.len();
             }
+            term.flush()?;
 
             loop {
                 match term.read_key()? {
                     Key::Backspace if position > 0 => {
                         position -= 1;
                         chars.remove(position);
-                        term.clear_chars(1)?;
+                        let line_size = term.size().1 as usize;
+                        // Case we want to delete last char of a line so the cursor is at the beginning of the next line
+                        if (position + prompt_len) % (line_size - 1) == 0 {
+                            term.clear_line()?;
+                            term.move_cursor_up(1)?;
+                            term.move_cursor_right(line_size + 1)?;
+                        } else {
+                            term.clear_chars(1)?;
+                        }
 
                         let tail: String = chars[position..].iter().collect();
 
                         if !tail.is_empty() {
                             term.write_str(&tail)?;
-                            term.move_cursor_left(tail.len())?;
+
+                            let total = position + prompt_len + tail.len();
+                            let total_line = total / line_size;
+                            let line_cursor = (position + prompt_len) / line_size;
+                            term.move_cursor_up(total_line - line_cursor)?;
+
+                            term.move_cursor_left(line_size)?;
+                            term.move_cursor_right((position + prompt_len) % line_size)?;
                         }
 
                         term.flush()?;
@@ -314,15 +342,112 @@ where
                         term.flush()?;
                     }
                     Key::ArrowLeft if position > 0 => {
-                        term.move_cursor_left(1)?;
+                        if (position + prompt_len) % term.size().1 as usize == 0 {
+                            term.move_cursor_up(1)?;
+                            term.move_cursor_right(term.size().1 as usize)?;
+                        } else {
+                            term.move_cursor_left(1)?;
+                        }
                         position -= 1;
                         term.flush()?;
                     }
                     Key::ArrowRight if position < chars.len() => {
-                        term.move_cursor_right(1)?;
+                        if (position + prompt_len) % (term.size().1 as usize - 1) == 0 {
+                            term.move_cursor_down(1)?;
+                            term.move_cursor_left(term.size().1 as usize)?;
+                        } else {
+                            term.move_cursor_right(1)?;
+                        }
                         position += 1;
                         term.flush()?;
                     }
+                    Key::UnknownEscSeq(seq) if seq == vec!['b'] => {
+                        let line_size = term.size().1 as usize;
+                        let nb_space = chars[..position]
+                            .iter()
+                            .rev()
+                            .take_while(|c| c.is_whitespace())
+                            .count();
+                        let find_last_space = chars[..position - nb_space]
+                            .iter()
+                            .rposition(|c| c.is_whitespace());
+
+                        // If we find a space we set the cursor to the next char else we set it to the beginning of the input
+                        if let Some(mut last_space) = find_last_space {
+                            if last_space < position {
+                                last_space += 1;
+                                let new_line = (prompt_len + last_space) / line_size;
+                                let old_line = (prompt_len + position) / line_size;
+                                let diff_line = old_line - new_line;
+                                if diff_line != 0 {
+                                    term.move_cursor_up(old_line - new_line)?;
+                                }
+
+                                let new_pos_x = (prompt_len + last_space) % line_size;
+                                let old_pos_x = (prompt_len + position) % line_size;
+                                let diff_pos_x = new_pos_x as i64 - old_pos_x as i64;
+                                //println!("new_pos_x = {}, old_pos_x = {}, diff = {}", new_pos_x, old_pos_x, diff_pos_x);
+                                if diff_pos_x < 0 {
+                                    term.move_cursor_left(-diff_pos_x as usize)?;
+                                } else {
+                                    term.move_cursor_right((diff_pos_x) as usize)?;
+                                }
+                                position = last_space;
+                            }
+                        } else {
+                            term.move_cursor_left(position)?;
+                            position = 0;
+                        }
+
+                        term.flush()?;
+                    }
+                    Key::UnknownEscSeq(seq) if seq == vec!['f'] => {
+                        let line_size = term.size().1 as usize;
+                        let find_next_space =
+                            chars[position..].iter().position(|c| c.is_whitespace());
+
+                        // If we find a space we set the cursor to the next char else we set it to the beginning of the input
+                        if let Some(mut next_space) = find_next_space {
+                            let nb_space = chars[position + next_space..]
+                                .iter()
+                                .take_while(|c| c.is_whitespace())
+                                .count();
+                            next_space += nb_space;
+                            let new_line = (prompt_len + position + next_space) / line_size;
+                            let old_line = (prompt_len + position) / line_size;
+                            term.move_cursor_down(new_line - old_line)?;
+
+                            let new_pos_x = (prompt_len + position + next_space) % line_size;
+                            let old_pos_x = (prompt_len + position) % line_size;
+                            let diff_pos_x = new_pos_x as i64 - old_pos_x as i64;
+                            if diff_pos_x < 0 {
+                                term.move_cursor_left(-diff_pos_x as usize)?;
+                            } else {
+                                term.move_cursor_right((diff_pos_x) as usize)?;
+                            }
+                            position += next_space;
+                        } else {
+                            let new_line = (prompt_len + chars.len()) / line_size;
+                            let old_line = (prompt_len + position) / line_size;
+                            term.move_cursor_down(new_line - old_line)?;
+
+                            let new_pos_x = (prompt_len + chars.len()) % line_size;
+                            let old_pos_x = (prompt_len + position) % line_size;
+                            let diff_pos_x = new_pos_x as i64 - old_pos_x as i64;
+                            match diff_pos_x.cmp(&0) {
+                                Ordering::Less => {
+                                    term.move_cursor_left((-diff_pos_x - 1) as usize)?;
+                                }
+                                Ordering::Equal => {}
+                                Ordering::Greater => {
+                                    term.move_cursor_right((diff_pos_x) as usize)?;
+                                }
+                            }
+                            position = chars.len();
+                        }
+
+                        term.flush()?;
+                    }
                     #[cfg(feature = "completion")]
                     Key::ArrowRight | Key::Tab => {
                         if let Some(completion) = &self.completion {
@@ -342,10 +467,27 @@ where
                     }
                     #[cfg(feature = "history")]
                     Key::ArrowUp => {
+                        let line_size = term.size().1 as usize;
                         if let Some(history) = &self.history {
                             if let Some(previous) = history.read(hist_pos) {
                                 hist_pos += 1;
-                                term.clear_chars(chars.len())?;
+                                let mut chars_len = chars.len();
+                                while ((prompt_len + chars_len) / line_size) > 0 {
+                                    term.clear_chars(chars_len)?;
+                                    if (prompt_len + chars_len) % line_size == 0 {
+                                        chars_len -= std::cmp::min(chars_len, line_size);
+                                    } else {
+                                        chars_len -= std::cmp::min(
+                                            chars_len,
+                                            (prompt_len + chars_len + 1) % line_size,
+                                        );
+                                    }
+                                    if chars_len > 0 {
+                                        term.move_cursor_up(1)?;
+                                        term.move_cursor_right(line_size)?;
+                                    }
+                                }
+                                term.clear_chars(chars_len)?;
                                 chars.clear();
                                 position = 0;
                                 for ch in previous.chars() {
@@ -359,7 +501,27 @@ where
                     }
                     #[cfg(feature = "history")]
                     Key::ArrowDown => {
+                        let line_size = term.size().1 as usize;
                         if let Some(history) = &self.history {
+                            let mut chars_len = chars.len();
+                            while ((prompt_len + chars_len) / line_size) > 0 {
+                                term.clear_chars(chars_len)?;
+                                if (prompt_len + chars_len) % line_size == 0 {
+                                    chars_len -= std::cmp::min(chars_len, line_size);
+                                } else {
+                                    chars_len -= std::cmp::min(
+                                        chars_len,
+                                        (prompt_len + chars_len + 1) % line_size,
+                                    );
+                                }
+                                if chars_len > 0 {
+                                    term.move_cursor_up(1)?;
+                                    term.move_cursor_right(line_size)?;
+                                }
+                            }
+                            term.clear_chars(chars_len)?;
+                            chars.clear();
+                            position = 0;
                             // Move the history position back one in case we have up arrowed into it
                             // and the position is sitting on the next to read
                             if let Some(pos) = hist_pos.checked_sub(1) {
@@ -367,35 +529,18 @@ where
                                 // Move it back again to get the previous history entry
                                 if let Some(pos) = pos.checked_sub(1) {
                                     if let Some(previous) = history.read(pos) {
-                                        term.clear_chars(chars.len())?;
-                                        chars.clear();
-                                        position = 0;
                                         for ch in previous.chars() {
                                             chars.insert(position, ch);
                                             position += 1;
                                         }
                                         term.write_str(&previous)?;
-                                        term.flush()?;
                                     }
-                                } else {
-                                    term.clear_chars(chars.len())?;
-                                    chars.clear();
-                                    position = 0;
                                 }
-                            } else {
-                                term.clear_chars(chars.len())?;
-                                chars.clear();
-                                position = 0;
                             }
+                            term.flush()?;
                         }
                     }
                     Key::Enter => break,
-                    Key::Unknown => {
-                        return Err(io::Error::new(
-                            io::ErrorKind::NotConnected,
-                            "Not a terminal",
-                        ))
-                    }
                     _ => (),
                 }
             }
@@ -438,7 +583,11 @@ where
                     }
 
                     if self.report {
-                        render.input_prompt_selection(&self.prompt, &input)?;
+                        if let Some(post_completion_text) = &self.post_completion_text {
+                            render.input_prompt_selection(post_completion_text, &input)?;
+                        } else {
+                            render.input_prompt_selection(&self.prompt, &input)?;
+                        }
                     }
                     term.flush()?;
 
